# Generated by Django 3.0 on 2019-12-18 12:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('olaii_stripe', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='paymentmethod',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
    ]



from decimal import Decimal

import stripe

from django.conf import settings

from .models import PaymentIntent, Customer, PaymentMethod, Refund

# Configure Stripe API Key
stripe.api_key = getattr(settings, 'STRIPE_API_KEY', None)


def generate_new_payment_intent(amount,
                                customer=None,
                                payment_method=None,
                                statement_descriptor_suffix=None,
                                setup_future_usage=None,
                                capture_method=PaymentIntent.CAPTURE_AUTOMATIC,
                                currency='eur'):
    """
    Create new Payment Intent based on parameters provided.

    Returns: PaymentIntent from database with populated fields.
    """
    intent_obj = stripe.PaymentIntent.create(
        amount=convert_to_stripe_amount(amount),
        currency=currency,
        customer=customer,
        payment_method=payment_method,
        setup_future_usage=setup_future_usage,
        capture_method=capture_method,
        statement_descriptor_suffix=statement_descriptor_suffix
    )

    # Store in database PaymentIntent
    intent = PaymentIntent.objects.create(stripe_id=intent_obj.id,
                                          amount=intent_obj.amount,
                                          amount_capturable=intent_obj.amount_capturable,
                                          amount_received=intent_obj.amount_received,
                                          currency=intent_obj.currency,
                                          customer=customer,
                                          payment_method=payment_method,
                                          description=intent_obj.description,
                                          capture_method=intent_obj.capture_method,
                                          client_secret=intent_obj.client_secret,
                                          setup_future_usage=intent_obj.setup_future_usage,
                                          statement_descriptor=intent_obj.statement_descriptor,
                                          statement_descriptor_suffix=intent_obj.statement_descriptor_suffix,
                                          status=intent_obj.status
                                          )

    # return Database Object back
    return intent


def create_stripe_customer(user):
    """
    Create Customer object on Stripe API from Django User object.
    """
    customer_obj = stripe.Customer.create(
        email=user.email,
        name=f"{user.first_name} {user.last_name}"
    )

    customer = Customer.objects.create(stripe_id=customer_obj.id,
                                       user=user,
                                       name=customer_obj.name,
                                       address=customer_obj.address,
                                       email=customer_obj.email)

    return customer


def attach_payment_method_to_customer(payment_method_stripe_id, customer):
    pm_object = stripe.PaymentMethod.attach(payment_method_stripe_id, customer=customer.stripe_id)

    payment_method = PaymentMethod.objects.create(stripe_id=pm_object.id,
                                                  customer=customer,
                                                  user=customer.user,
                                                  brand=pm_object.card.brand,
                                                  country=pm_object.card.country,
                                                  funding=pm_object.card.funding,
                                                  exp_month=pm_object.card.exp_month,
                                                  exp_year=pm_object.card.exp_year,
                                                  last4=pm_object.card.last4,
                                                  fingerprint=pm_object.card.fingerprint,
                                                  supports_3d_secure=pm_object.card.three_d_secure_usage.supported)

    return payment_method


def refund_payment_intent(payment_intent, amount):
    refund_obj = stripe.Refund.create(
        amount=convert_to_stripe_amount(amount),
        payment_intent=payment_intent.stripe_id
    )

    refund = Refund(
        stripe_id=refund_obj.id,
        payment_intent=payment_intent,
        amount=refund_obj.amount,
    )

    return refund


def convert_to_stripe_amount(amount):
    # Convert 1.00€ to 100 for different types
    if type(amount) == Decimal:
        return int(f"{amount * Decimal(100):.0f}")
    else:
        return int(f"{Decimal(amount) * Decimal(100):.0f}")



from django.contrib import admin

from .models import Customer, PaymentMethod, PaymentIntent


class CustomerAdmin(admin.ModelAdmin):
    list_display = ['stripe_id', 'name', 'email']
    search_fields = ['name', 'email', 'stripe_id']


class PaymentMethodAdmin(admin.ModelAdmin):
    list_display = ['stripe_id', 'last4', 'customer', 'brand', 'country']
    search_fields = ['customer__name', 'customer__email', 'stripe_id', 'last4']


class PaymentIntentAdmin(admin.ModelAdmin):
    list_display = ['stripe_id', 'status', 'amount']
    search_fields = ['stripe_id', 'payment_method__customer__email']


admin.site.register(Customer, CustomerAdmin)
admin.site.register(PaymentMethod, PaymentMethodAdmin)
admin.site.register(PaymentIntent, PaymentIntentAdmin)



import stripe

from django.conf import settings

from .models import PaymentIntent, PaymentMethod
from .helpers import attach_payment_method_to_customer
from .exceptions import StripeEventException, StripeWrongEventTypeException

# Configure Stripe API Key
stripe.api_key = getattr(settings, 'STRIPE_API_KEY', None)


def construct_webhook_event(payload, sig_header, endpoint_secret):
    """
    Construct webhook event
    """
    try:

        return stripe.Event.construct_from(
            payload,
            sig_header,
            endpoint_secret)
    except ValueError as e:
        raise StripeEventException(e)


def process_payment_intent_succeeded_webhook(payload, sig_header, endpoint_secret):
    """
    Process Payment Intent Succeeded Webhook event.

    It will update amount capturable, received, status and attach a Payment Method to a Customer instance if
    it's a new payment method.
    """
    event = construct_webhook_event(payload, sig_header, endpoint_secret)

    if event.type != 'payment_intent.succeeded':
        raise StripeWrongEventTypeException('Event is not of type payment_intent.succeeded')

    intent_obj = event.data.object  # contains a stripe.PaymentIntent object
    intent = PaymentIntent.objects.get(stripe_id=intent_obj.id)

    # Get Payment method Stripe ID
    payment_method_stripe_id = intent_obj.payment_method

    qs = PaymentMethod.objects.filter(stripe_id=payment_method_stripe_id)
    # Attach Payment Method to Customer object if it doesn't exist in the system
    if not qs.exists():
        payment_method = attach_payment_method_to_customer(payment_method_stripe_id, intent.customer)
        intent.payment_method = payment_method
    else:
        intent.payment_method = qs.first()

    intent.amount_capturable = intent_obj.amount_capturable
    intent.amount_received = intent_obj.amount_received
    intent.status = intent_obj.status
    intent.save()

    return intent



from django.db import models
from django.conf import settings


class Customer(models.Model):
    stripe_id = models.CharField(max_length=255)

    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, related_name='stripe_customer')

    name = models.CharField(max_length=255, null=True, blank=True)

    address = models.CharField(max_length=255, null=True, blank=True)
    email = models.EmailField(null=True, blank=True)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"[{self.stripe_id}] {self.email}"


class PaymentMethod(models.Model):
    FUNDING_CREDIT = 'credit'
    FUNDING_DEBIT = 'debit'
    FUNDING_PREPAID = 'prepaid'
    FUNDING_UNKNOWN = 'unknown'

    FUNDING_OPTIONS = [
        (FUNDING_CREDIT, 'Credit'),
        (FUNDING_DEBIT, 'Debit'),
        (FUNDING_PREPAID, 'Prepaid'),
        (FUNDING_UNKNOWN, 'Unknown'),
    ]

    stripe_id = models.CharField(max_length=255)

    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, related_name='stripe_cards')
    customer = models.ForeignKey(Customer, null=True, blank=True, on_delete=models.SET_NULL)

    # Card Details
    brand = models.CharField(max_length=255)
    country = models.CharField(max_length=255, blank=True, null=True)
    funding = models.CharField(max_length=255, choices=FUNDING_OPTIONS, default=FUNDING_UNKNOWN)

    exp_month = models.PositiveIntegerField(default=0)
    exp_year = models.PositiveIntegerField(default=0)
    last4 = models.CharField(max_length=255)

    fingerprint = models.CharField(max_length=255)
    supports_3d_secure = models.BooleanField()

    deleted = models.BooleanField(default=False)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"[{self.stripe_id}] {self.last4}"


class PaymentIntent(models.Model):
    CAPTURE_AUTOMATIC = 'automatic'
    CAPTURE_MANUAL = 'manual'

    CAPTURE_OPTIONS = [
        (CAPTURE_AUTOMATIC, 'Automatic'),
        (CAPTURE_MANUAL, 'Manual'),
    ]

    FUTURE_ON_SESSION = 'on_session'
    FUTURE_OFF_SESSION = 'off_session'

    FUTURE_USAGE_OPTIONS = [
        (FUTURE_ON_SESSION, 'On Session'),
        (FUTURE_OFF_SESSION, 'Off Session'),
    ]

    STATUS_REQUIRES_PAYMENT_METHOD = 'requires_payment_method'
    STATUS_REQUIRES_CONFIRMATION = 'requires_confirmation'
    STATUS_REQUIRES_ACTION = 'requires_action'
    STATUS_PROCESSING = 'processing'
    STATUS_REQUIRES_CAPTURE = 'requires_capture'
    STATUS_CANCELED = 'canceled'
    STATUS_SUCCEEDED = 'succeeded'

    STATUS_OPTIONS = [
        (STATUS_REQUIRES_PAYMENT_METHOD, 'Requires Payment Method'),
        (STATUS_REQUIRES_CONFIRMATION, 'Requires Confirmation'),
        (STATUS_REQUIRES_ACTION, 'Requires Action'),
        (STATUS_PROCESSING, 'Processing'),
        (STATUS_REQUIRES_CAPTURE, 'Requires Capture'),
        (STATUS_CANCELED, 'Canceled'),
        (STATUS_SUCCEEDED, 'Succeeded')
    ]

    stripe_id = models.CharField(max_length=255)

    amount = models.PositiveIntegerField(default=0)
    amount_capturable = models.PositiveIntegerField(default=0)
    amount_received = models.PositiveIntegerField(default=0)
    currency = models.CharField(max_length=3, default='eur')

    customer = models.ForeignKey(Customer, null=True, blank=True, on_delete=models.SET_NULL, related_name='payment_intents')
    payment_method = models.ForeignKey(PaymentMethod, null=True, blank=True, on_delete=models.SET_NULL)

    description = models.TextField(blank=True, null=True)

    canceled_at = models.DateTimeField(null=True, blank=True)
    cancellation_reason = models.CharField(max_length=255, null=True, blank=True)

    capture_method = models.CharField(max_length=255, choices=CAPTURE_OPTIONS, default=CAPTURE_AUTOMATIC)

    client_secret = models.CharField(max_length=255)

    setup_future_usage = models.CharField(max_length=255, null=True, blank=True, choices=FUTURE_USAGE_OPTIONS, default=None)

    # Only for non-card payments
    statement_descriptor = models.CharField(max_length=22, null=True, blank=True)
    # For Card payments
    statement_descriptor_suffix = models.CharField(max_length=22, null=True, blank=True)

    status = models.CharField(max_length=255, choices=STATUS_OPTIONS, default=STATUS_REQUIRES_PAYMENT_METHOD)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"[{self.stripe_id}] {self.amount} {self.currency}"


class Refund(models.Model):
    stripe_id = models.CharField(max_length=255)
    payment_intent = models.ForeignKey('PaymentIntent', null=True, blank=True, on_delete=models.SET_NULL, related_name='refunds')

    amount = models.PositiveIntegerField(default=0)

    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f"[{self.stripe_id}] {self.amount}"



# Generated by Django 3.0 on 2019-12-16 14:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=255)),
                ('name', models.CharField(blank=True, max_length=255, null=True)),
                ('address', models.CharField(blank=True, max_length=255, null=True)),
                ('email', models.EmailField(blank=True, max_length=254, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='stripe_customer', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='PaymentMethod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=255)),
                ('brand', models.CharField(max_length=255)),
                ('country', models.CharField(blank=True, max_length=255, null=True)),
                ('funding', models.CharField(choices=[('credit', 'Credit'), ('debit', 'Debit'), ('prepaid', 'Prepaid'), ('unknown', 'Unknown')], default='unknown', max_length=255)),
                ('exp_month', models.PositiveIntegerField(default=0)),
                ('exp_year', models.PositiveIntegerField(default=0)),
                ('last4', models.CharField(max_length=255)),
                ('fingerprint', models.CharField(max_length=255)),
                ('supports_3d_secure', models.BooleanField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('customer', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='olaii_stripe.Customer')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='stripe_cards', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='PaymentIntent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=255)),
                ('amount', models.PositiveIntegerField(default=0)),
                ('amount_capturable', models.PositiveIntegerField(default=0)),
                ('amount_received', models.PositiveIntegerField(default=0)),
                ('currency', models.CharField(default='eur', max_length=3)),
                ('description', models.TextField(blank=True)),
                ('canceled_at', models.DateTimeField(blank=True, null=True)),
                ('cancellation_reason', models.CharField(blank=True, max_length=255, null=True)),
                ('capture_method', models.CharField(choices=[('automatic', 'Automatic'), ('manual', 'Manual')], default='automatic', max_length=255)),
                ('client_secret', models.CharField(max_length=255)),
                ('setup_future_usage', models.CharField(blank=True, choices=[('on_session', 'On Session'), ('off_session', 'Off Session')], default=None, max_length=255, null=True)),
                ('statement_descriptor', models.CharField(blank=True, max_length=22, null=True)),
                ('statement_descriptor_suffix', models.CharField(blank=True, max_length=22, null=True)),
                ('status', models.CharField(choices=[('requires_payment_method', 'Requires Payment Method'), ('requires_confirmation', 'Requires Confirmation'), ('requires_action', 'Requires Action'), ('processing', 'Processing'), ('requires_capture', 'Requires Capture'), ('canceled', 'Canceled'), ('succeeded', 'Succeeded')], default='requires_payment_method', max_length=255)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('customer', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='payment_intents', to='olaii_stripe.Customer')),
                ('payment_method', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='olaii_stripe.PaymentMethod')),
            ],
        ),
    ]



# Generated by Django 3.0 on 2020-01-06 10:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('olaii_stripe', '0003_auto_20191227_1001'),
    ]

    operations = [
        migrations.CreateModel(
            name='Refund',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stripe_id', models.CharField(max_length=255)),
                ('amount', models.PositiveIntegerField(default=0)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('payment_intent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='refunds', to='olaii_stripe.PaymentIntent')),
            ],
        ),
    ]



# Generated by Django 3.0.1 on 2019-12-27 10:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('olaii_stripe', '0002_paymentmethod_deleted'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentintent',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]
